@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2009 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,9 @@ var gFlashGot = {
     ev.currentTarget.removeEventListener(ev.type, arguments.callee, false);
     try {
       gFlashGot.init();
+      
+      if (!gFlashGotService._initialized) window.setTimeout(function() { gFlashGotService.init(); }, 500);
+      
     } catch(e) {
       dump("FlashGot init error: " + e.message);
       gFlashGot.log("Unrecoverable init error: " + e.message + " --- " + e.stack);
@@ -45,25 +48,30 @@ var gFlashGot = {
     gFlashGotService.dom._winType = document.documentElement.getAttribute("windowtype");
     
     // install listeners
-    gFlashGot.isMouseDown = false;
+    gFlashGot.mouseDown = null;
     var target = window.gBrowser || window;
     
     if (target.tabContainer) {
         target.tabContainer.addEventListener("TabSelect", this.updateMediaStatus, false);
     }
-    
-    
-    
+
     target.addEventListener("load", this.updateMediaStatus, true);
     
     target.addEventListener("mousedown", function(ev) {
-      if (!gFlashGot._isContentEvent(ev)) return;
-      gFlashGot.isMouseDown = true;
+      if (!(ev.button <= 1 && gFlashGot._isContentEvent(ev))) return;
+      gFlashGot.mouseDown = { x: ev.screenX, y: ev.screenY };
       gFlashGot.hoverElement = ev.originalTarget;
     }, false);
     
+    target.addEventListener("mousemove", function(ev) {
+      const md = gFlashGot.mouseDown;
+      if (md && md.y && ev.screenY < md.y) {
+        md.y = 0;
+      }
+    }, false);
+    
     target.addEventListener("mouseover", function(ev) {
-      if(!gFlashGot.isMouseDown) gFlashGot.hoverElement = ev.originalTarget;
+      if (!gFlashGot.mouseDown) gFlashGot.hoverElement = ev.originalTarget;
     }, false);
     
     target.addEventListener("submit", function(ev) {
@@ -75,39 +83,57 @@ var gFlashGot = {
     }, false);
 
     target.addEventListener("mouseup", function(ev) {
-      gFlashGot.isMouseDown = false;
+      const md = gFlashGot.mouseDown;
+      const button = ev.button;
+      gFlashGot.mouseDown = null;
       
-      if (!gFlashGot._isContentEvent(ev)) return;
+      if (!(button <= 1 && gFlashGot._isContentEvent(ev) && gFlashGotService.interceptor)
+          || ev.ctrlKey || ev.metaKey)
+        return;
       
       gFlashGotService.interceptor.bypassAutoStart = false;
       gFlashGotService.interceptor.forceAutoStart = false;
-      if(ev.altKey && !(ev.ctrlKey || ev.metaKey)) {
+      
+      var trigger = false;
+      if (button === 0) {
+        trigger = ev.altKey;
+      } else if (md.y) {
+        var dy = ev.screenY - md.y;
+        var dx = Math.abs(ev.screenX - md.x);
+        trigger = dy > 16 && dx < 16 && dx / dy < .5; // south gesture
+      }
+      
+      if(trigger) {
         function prevent() {
           ev.preventDefault();
           ev.stopPropagation();
           gFlashGot.lastClickCaptureTime = new Date().getTime();
         }
-        var invert = gFlashGotService.getPref("invertAltShiftClick", false);
-        if((ev.shiftKey && !invert) || (invert && !ev.shiftKey)) {
-          gFlashGotService.interceptor.bypassAutoStart =
-            gFlashGotService.getPref("bypassCombo", true);
-          return;
-        } else {
-          if(gFlashGotService.getPref("altClick")) {
-            try {
-              if(gFlashGot.download()) {
-                prevent();
-                return;
-              }
-            } catch(ex) {}
-            gFlashGotService.interceptor.forceAutoStart = true;
-          } else {
+        
+        if (button === 0) {
+          var invert = gFlashGotService.getPref("invertAltShiftClick", false);
+          if((ev.shiftKey && !invert) || (invert && !ev.shiftKey)) {
+            gFlashGotService.interceptor.bypassAutoStart =
+              gFlashGotService.getPref("bypassCombo", true);
             return;
           }
         }
         
+        if(gFlashGotService.getPref(button === 0 ? "altClick" : "gesture")) {
+          try {
+            if(gFlashGot.download()) {
+              prevent();
+              return;
+            }
+            if (button !== 0) return;
+          } catch(ex) {}
+          gFlashGotService.interceptor.forceAutoStart = true;
+        } else {
+          return;
+        }
+        
         ev2 = ev.view.document.createEvent("MouseEvents");
-        ev2.initMouseEvent("click",ev.canBubble, ev.cancelable, 
+        ev2.initMouseEvent("click", ev.canBubble, ev.cancelable, 
                            ev.view, ev.detail, ev.screenX, ev.screenY, 
                            ev.clientX, ev.clientY, 
                            //ev.ctrlKey, ev.altKey, ev.shiftKey, ev.metaKey,
@@ -140,37 +166,28 @@ var gFlashGot = {
         }
     }, false);
     
-    this.toggleMainMenuIcon();
     
-    const prevVer = gFlashGotService.getPref("version", "");
-    if(prevVer != gFlashGotService.VERSION) {
-      gFlashGotService.setPref("version", gFlashGotService.VERSION);
-      gFlashGotService.savePrefs();
-      if(gFlashGotService.getPref("firstRunRedirection", true)) {
-        window.setTimeout(function() {
-          const url = "http://flashgot.net?ver=" + gFlashGotService.VERSION + "&prev=" + prevVer;
-          const browser = getBrowser();
-          browser.selectedTab = browser.addTab(url, null);
-        }, 500);
-      }
-    }
-    
-    if(typeof(flashgot_doDirtyJobs) == "function") {
-      window.setTimeout(flashgot_doDirtyJobs,0);
-    }
+    window.setTimeout(function() {
+      gFlashGotService.checkVersion();
+      if (gFlashGotService.firstRun) gFlashGot._mediaUI; // forces relocation
+      
+      gFlashGot.toggleMainMenuIcon();
+    }, 500);
   }
+
 , 
   log: function(msg) {
     gFlashGotService.log(msg);
   }
 ,
-  _contextMenu: null,
   get contextMenu() {
-    var cm=this._contextMenu;
-    if(!cm) {
-      this._contextMenu = cm = 
-        document.getElementById("contentAreaContextMenu") || 
-        document.getElementById("messagePaneContext")
+    var cm =
+        document.getElementById("contentAreaContextMenu") ||
+        document.getElementById("mailContext") || // TB3
+        document.getElementById("messagePaneContext"); // TB2 
+    if (cm) {
+      delete this.contextMenu;
+      this.contextMenu = cm;
     }
     return cm;
   }
@@ -179,9 +196,12 @@ var gFlashGot = {
     gFlashGotService.setPref(opt, !gFlashGotService.getPref(opt));
   }
 ,
-  openOptionsDialog: function() {
-    window.open("chrome://flashgot/content/flashgotOptions.xul", "flashgotOptions",
-      "chrome,dialog,centerscreen,alwaysRaised");
+  openOptionsDialog: function(tab) {
+     window.openDialog(
+        "chrome://flashgot/content/flashgotOptions.xul",
+        "flashgotOptions",
+        "chrome, dialog, centerscreen, alwaysRaised",
+        arguments.length ? {tabselIndexes:  [tab || 0]} : null);  
   }
 ,
   openAboutDialog: function() {
@@ -191,7 +211,7 @@ var gFlashGot = {
 ,
   browse: function(url) {
     var browser =  window.getBrowser();
-    browser.selectedTab = browser.addTab(url, null);
+    browser.selectedTab = browser.addTab(url);
   }
 ,
   browseHomePage: function() {
@@ -217,7 +237,7 @@ var gFlashGot = {
   }
 ,
   toggleMainMenuIcon: function() {
-    this.toggleIcon(document.getElementById("flashgot-menu"),this.hideIcons);
+    this.toggleIcon(document.getElementById("flashgot-menu"), this.hideIcons);
   }
 ,
   prepareToolsMenu: function(ev) {
@@ -233,7 +253,7 @@ var gFlashGot = {
     
     
     if(gFlashGotService && !(document.getElementById("flashgot-menu").hidden = gFlashGotService.getPref("hide-menu"))) {
-      const dis = !gFlashGotService.DMS.found;
+      const dis = false; // !gFlashGotService.DMS.found; // can never be since we implemented built-in
       const hideIcons = this.hideIcons;
       this.updateMediaUI();
       
@@ -242,19 +262,19 @@ var gFlashGot = {
       toggleMenu("sel", dis || this.isSelInvalid);
       toggleMenu("buildGallery", false);
       toggleMenu("media", dis || !this.media);
-      toggleMenu("opts", false);
-      this.prepareOptsMenu(ev.target);
-      
+      toggleMenu("opts", false);      
     }
   }
 ,
-  
+ 
   prepareContextMenu: function(ev) {
-    
     this.toggleMainMenuIcon();
+    
+    const fg = gFlashGotService;
     var menuCount = 0;
    
     function menuSwitch(name, disabled) {
+      
       var menuItem = document.getElementById("flashgot-menuitem-" + name);
       
       if(menuItem && 
@@ -268,28 +288,24 @@ var gFlashGot = {
        gFlashGot.toggleIcon(menuItem, hideIcons);
       }
     }
-    var menuItem = null;
-    const defaultDM = gFlashGotService.defaultDM;  
-    const dms = gFlashGotService.DMS;
-    var dm = dms.found ? dms[defaultDM] : null;
     
-    gFlashGotService.restoreNativeUIs(document);
-    dm.hideNativeUI(document);
-   
-    const invalidLink = !this.popupLink;
-   
+    var menuItem = null;
+    const defaultDM = fg.defaultDM;  
+    const dms = fg.hasDMS ? fg.DMS : null;
+    var dm = dms && (dms.found ? dms[defaultDM] : null) || {hideNativeUI: function() {}};
+    
+    const invalidLink = !this.popupLink;   
     const invalidSel = this.isSelInvalid;
     
     const noLink = this.linksCount == 0;
-    const hideDisabled = gFlashGotService.getPref("hideDisabledCmds");
+    const hideDisabled = fg.getPref("hideDisabledCmds");
    
-    
-    var hidden = !(dms.found && dm);
+    var hidden = !dm;
     const hideIcons = this.hideIcons;
     
     var hideLink = invalidLink || hidden || dm.disabledLink;
     var hideSel = invalidSel || hidden  || dm.disabledSel;
-    this.prepareCommandsMenu(document.getElementById("flashgot-menuitem-it"), hideLink && hideSel);
+    
     
     this.updateMediaUI();
     
@@ -317,9 +333,9 @@ var gFlashGot = {
     const sep1 = document.getElementById("flashgot-context-separator");
     const sep2 = document.getElementById("flashgot-context-separator2");
     const menu = sep1.parentNode;
-    var next=null;
+    var next = null;
     
-    const nested = gFlashGotService.getPref("nested-menu") && (menuCount > 1);
+    const nested = fg.getPref("nested-menu") && (menuCount > 1);
     submenu.hidden = !nested;
     if(nested) {
       menuCount=0;
@@ -327,11 +343,11 @@ var gFlashGot = {
         menuItem = document.getElementById("flashgot-menuitem-tabs");
         if (menuItem)
             menuItem.setAttribute("accesskey",
-                document.getElementById("flashgot-main-menuitem-tabs").getAttribute("acceskey"));
+                document.getElementById("flashgot-main-menuitem-tabs").getAttribute("accesskey"));
         for(menuItem = sep1.nextSibling;
             menuItem && (menuItem != sep2); 
             menuItem = next) {
-          next=menuItem.nextSibling;
+          next = menuItem.nextSibling;
           subpop.appendChild(menuItem);
         }
       }
@@ -339,11 +355,23 @@ var gFlashGot = {
       menuItem = document.getElementById("flashgot-menuitem-tabs");
       if (menuItem) menuItem.removeAttribute("accesskey");
       for(menuItem = subanchor.nextSibling; menuItem; menuItem = next) {
-        next=menuItem.nextSibling;
+        next = menuItem.nextSibling;
         menu.insertBefore(menuItem, sep2);
       }
     }
     sep1.hidden = menuCount == 0;
+    
+    if (!fg.hasDMS) {
+      ev.target.addEventListener("popupshown", function shown(ev) {
+        ev.currentTarget.removeEventListener(ev.type, shown, false);
+        if (fg.DMS) gFlashGot.prepareContextMenu(ev);
+      }, false);
+    } else {
+      fg.restoreNativeUIs(document);
+      dm.hideNativeUI(document);
+      this.prepareCommandsMenu(document.getElementById("flashgot-menuitem-it"), hideLink && hideSel);
+    }
+    
   }
 ,
   disposeContextMenu: function(ev) {
@@ -351,22 +379,32 @@ var gFlashGot = {
   },
   prepareCommandsMenu: function(anchorNode, hideOnly) {
     var node, parentNode = anchorNode.parentNode;
-    while((node = anchorNode.previousSibling) && /\bflashgot-command-menuitem\b/.test(node.className)) {
-      parentNode.removeChild(node);
-    }
+    
+    var cmi = (parentNode.getElementsByClassName && parentNode.getElementsByClassName("flashgot-command-menuitem")
+               || parentNode.getElementsByTagName("menuitem"));
+    
+    var j, len;
+    
+    for(j = cmi.length; j-- > 0;) 
+      if (/^flashgot-command-mi-/.test(cmi[j].id))
+        parentNode.removeChild(cmi[j]);
+    
     if(hideOnly) return;
     
-    var mi, dm;
+    var mi, dm, id;
     const dms = gFlashGotService.DMS;
-    for(var j = 0, len = dms.length; j < len; j++) {
+    for(j = 0, len = dms.length; j < len; j++) {
       dm = dms[j];
       if(dm.supported && dm.shownInContextMenu) {
-        mi = document.createElement("menuitem");
-        mi.setAttribute("class", "menuitem-iconic flashgot-command-menuitem");
-        mi.setAttribute("label", dm.name);
-        mi.setAttribute("id", "flashgot-command-mi-" + dm.codeName);
-        mi.setAttribute("oncommand", "gFlashGot.downloadSel(this.label) || gFlashGot.downloadPopupLink(this.label)");
-        parentNode.insertBefore(mi, anchorNode);
+        id =  "flashgot-command-mi-" + dm.codeName;
+        if (!document.getElementById(id)) {
+          mi = document.createElement("menuitem");
+          mi.setAttribute("class", "menuitem-iconic flashgot-command-menuitem");
+          mi.setAttribute("label", dm.name);
+          mi.setAttribute("id", id);
+          mi.setAttribute("oncommand", "gFlashGot.downloadSel(this.label) || gFlashGot.downloadPopupLink(this.label)");
+          parentNode.insertBefore(mi, anchorNode);
+        }
         dm.hideNativeUI(document);
       }
     }
@@ -447,20 +485,60 @@ var gFlashGot = {
     }
   }
 ,
-  
-  
-  
-  _mediaUI: null,
-  updateMediaUI: function() {
-    if (!this._mediaUI) {
-      this._mediaUI = ["flashgot-menuitem-media", "flashgot-main-menuitem-media","flashgot-media-status"]
-        .map(function(id) { return document.getElementById(id); });
+
+  get _mediaUI() {
+    var addonBar = document.getElementById("addon-bar");
+    var widgetId = "flashgot-media-status";
+    if (addonBar) {
+      var el = document.getElementById(widgetId);
+      if (el) el.parentNode.removeChild(el);
+      widgetId = "flashgot-media-tbb";
+      el = document.getElementById(widgetId);
+      if (el && el.nextSibling && el.nextSibling.id == "search-container") {
+        el = null;
+      }
+      if (!el) {
+        var bar, refId;
+        var navBar = document.getElementById("nav-bar");
+        if(addonBar.collapsed && navBar && !navBar.collapsed) {
+          bar = navBar;
+          refId = "urlbar-container";
+        } else {
+          bar = addonBar;
+          refId = "status";
+        }
+        
+        var set = bar.currentSet.split(/\s*,\s*/);
+        
+        for (var p; (p = set.indexOf(widgetId)) > -1;)
+          set.splice(p, 1);
+          
+        set.splice(set.indexOf(refId), 0, widgetId); 
+        bar.setAttribute("currentset", bar.currentSet = set.join(","));
+        document.persist(bar.id, "currentset");
+        try {
+          window.BrowserToolboxCustomizeDone(true);
+        } catch (e) {}
+      }
     }
-    var mm = this.media;
-    var count = (mm && mm.length) ? " (" + mm.length + ")" : "";
-    var tip = (count && typeof(/ /) == "object") // Fx >= 3, multiline tooltips supported 
-        ? mm.map(function(l) { return l.tip }).join("\n")
-        : '';
+    delete this._mediaUI;
+    return this._mediaUI =
+      [widgetId, "flashgot-menuitem-media", "flashgot-main-menuitem-media"]
+        .map(function(id) { return document.getElementById(id) })
+        .filter(function(el) { return !!el });
+  },
+  
+  _mediaTip: function(media) {
+    return media && media.length &&
+      typeof(/ /) === "object" // Fx >= 3, multiline tooltips supported 
+      ? media.map(function(l) { return l.tip }).join("\n")
+      : '';
+  },
+  
+  updateMediaUI: function() {
+    var media = this.media;
+    var count = (media && media.length) ? " (" + media.length + ")" : "";
+    var tip = this._mediaTip(media);
     
     var l;
     for each(var ui in this._mediaUI) {
@@ -478,18 +556,28 @@ var gFlashGot = {
   },
   
   updateMediaStatus: function(anim) {
-    if (this != gFlashGot) gFlashGot.updateMediaStatus();
-    else {
-      var ms = this.mediaStatusAnim.widget;
-      if (ms) {
-        if (gFlashGotService.getPref("media.statusIcon", true)) {
-            var m = this.media;
-            ms.hidden = ms.disabled = !(m && m.length);
-        } else {
-            ms.hidden = true;
-        }
-        this.mediaStatusAnim.run(!anim);
+    if (this !== gFlashGot) {
+      gFlashGot.updateMediaStatus();
+      return;
+    }
+    
+    var ms = this.mediaStatusAnim.widget;
+    if (ms) {
+      if (gFlashGotService.getPref("media.statusIcon", true)) {
+          
+          var media = this.media;
+          ms.hidden = ms.disabled = !(media && media.length);
+          
+          if (!(anim || ms.hidden)) {
+            var tip = this._mediaTip(media);
+            if (tip) ms.setAttribute("tooltiptext", tip);
+            else ms.removeAttribute("tooltiptext");
+          }
+          
+      } else {
+        ms.hidden = true;
       }
+      this.mediaStatusAnim.run(!anim);
     }
   },
   
@@ -499,7 +587,7 @@ var gFlashGot = {
     cycles: 0,
     get widget() {
       delete this.widget;
-      return this.widget = document.getElementById("flashgot-media-status");  
+      return this.widget = gFlashGot._mediaUI[0];  
     },
     run: function(once) {
       var w = this.widget;
@@ -528,12 +616,11 @@ var gFlashGot = {
   },
   
   prepareMediaMenu: function(menu) {
-    while (menu.firstChild) menu.removeChild(menu.firstChild);
+    var pivot = Array.slice(menu.getElementsByTagName("menuseparator")).pop();
+    while (pivot.nextSibling) menu.removeChild(pivot.nextSibling);
     var m = this.media;
     if (!(m && m.length)) return false;
-    
-    var generateListener = function(link) { return  };
-    
+  
     var mi;
     for (var j = 0; j < m.length; j++) {
       menu.appendChild(this._createMediaMenuItem(m[j]));
@@ -542,7 +629,7 @@ var gFlashGot = {
   },
   _createMediaMenuItem: function(l) {
     mi = document.createElement("menuitem");
-    mi.setAttribute("label", l.fname || l.tip);
+    mi.setAttribute("label", l.label);
     mi.setAttribute("tooltiptext", l.tip);
     mi.addEventListener("command", function() { gFlashGot.downloadMedia([l]); }, false);
     return mi;
@@ -559,7 +646,7 @@ var gFlashGot = {
 ,
 
   get isTabbed() {
-    var b = gBrowser;
+    var b = window.gBrowser;
     return b && b.browsers && b.browsers.length > 1;
   }
 ,
@@ -729,7 +816,7 @@ var gFlashGot = {
 
     // text links detection
     m = selString.match(
-      /\b(h.{2}p:\/\/|ftp:\/\/|mms:\/\/|rtsp:\/\/|[a-z]+[a-z\d]+\.[a-z\d\-\.]+\/)[^\s]*/gi);
+      /\b(?:(?:h.{2}p|ftp|mms|ed2k|rtsp|rtmpe?):\/\/|[a-z]+[a-z\d]+\.[a-z\d\-\.]+\/|magnet:\?)[^\s]*/gi);
     delete selString;
     var j, k;
     if(m) {
@@ -773,7 +860,7 @@ var gFlashGot = {
   }
 ,
   checkLink: function(link) {
-    return link.href && /^[a-z]+:\/\/.*/i.test(link.href) && !/^(javascript|mailto|news|file|data):/i.test(link.href);
+    return link.href && /^(?:\w+:\/\/.*|javascript:.*http|magnet:)/i.test(link.href) && !/^(mailto|news|file|data):/i.test(link.href);
   }
 ,
   _wrapAnchor: function(node, lm) {
@@ -824,6 +911,14 @@ var gFlashGot = {
     const link = this.popupLink;
     return link && this.download([link], gFlashGotService.OP_ONE, dmName);
   }
+,
+  downloadPopupNodeText: function(dmName) {
+    if (!(document.popupNode && document.popupNode.textContent)) return false;
+    return this.download([{
+      href: document.popupNode.textContent,
+      description: "Thunderbird Link"
+    }], gFlashGotService.OP_ONE, dmName);
+  }
 ,
   downloadLink: function(dmName) {
     const sel=this.srcWindow.getSelection();
@@ -872,7 +967,7 @@ var gFlashGot = {
   }
 ,
   get media() {
-    return window.content._flashgotMedia || null;
+    return "_flashgotMedia" in content ? content._flashgotMedia : null;
   },
   
   addMediaLinks: function(links, mm) {
@@ -881,6 +976,14 @@ var gFlashGot = {
     if (!(links.length || links.referrer)) links.referrer = mm[0].referrer;
     Array.prototype.push.apply(links, mm);
     return true;
+  },
+  
+  clearMedia: function() {
+    if (this.media) {
+       // keep _map in place, so we still prevent duplicates
+      this.media.length = 0;
+    }
+    this.updateMediaUI();
   }
 ,
   downloadAll: function(dmName) {
@@ -906,7 +1009,7 @@ var gFlashGot = {
   downloadMedia: function(mediaLinks, dmName) {
     var links = [];
     return this.addMediaLinks(links, mediaLinks) &&
-      this.download(links, gFlashGotService.OP_SEL, dmName);
+      this.download(links, gFlashGotService.OP_SEL, dmName || gFlashGotService.getPref("media.dm", ""));
   }
 ,
   download: function(links, opType, dmName) {
@@ -1039,7 +1142,4 @@ var gFlashGot = {
 
 }
 
-window.addEventListener("load", gFlashGot.onload, false);
-
-
-
+window.addEventListener("load", gFlashGot.onload, false);
\ No newline at end of file
