@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2009 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,13 +28,11 @@ const NS_BINDING_ABORTED = 0x804b0002;
 const EXTENSION_ID = "{19503e42-ca3c-4c27-b1e2-9cdb2170ee34}";
 const EXTENSION_NAME = "FlashGot";
 const CHROME_NAME = "flashgot";
-const VERSION = "1.2";
+const VERSION = "1.3.0.5";
 const SERVICE_NAME = EXTENSION_NAME + " Service";
 const SERVICE_CTRID = "@maone.net/flashgot-service;1";
 const SERVICE_ID = "{2a55fc5c-7b31-4ee1-ab15-5ee2eb428cbe}";
     
-const SERVICE_CONSTRUCTOR = FlashGotService;
-
 // interfaces implemented by this component
 const SERVICE_IIDS = 
 [ 
@@ -96,24 +94,39 @@ const INCLUDE = function(name) {
   }
 }
 
-INCLUDE('XPCOM', 'DOM', 'DMS', 'HttpInterceptor', 'MediaSniffer');
-
-const SHUTDOWN = "profile-before-change";
-const STARTUP = "profile-after-change";
-
-var fg, singleton; // singleton
-function FlashGotService() {
-  this.wrappedJSObject = fg = singleton = this;
-  
-  if ("nsIChromeRegistrySea" in CI) INCLUDE("SMUninstaller");
-  
-  const os = this.observerService;
-  os.addObserver(this, SHUTDOWN, false);
-  os.addObserver(this, "xpcom-shutdown", false);
-  os.addObserver(this, STARTUP, false);
+function LAZY_INCLUDE(name) {
+  if (arguments.length > 1)
+    for (var j = 0, len = arguments.length; j < len; j++)
+      arguments.callee(arguments[j]);
+  else {
+    __defineGetter__(name, function() {
+      delete this[name];
+      INCLUDE(name);
+      return this[name];
+    });
+  }
 }
 
-FlashGotService.prototype = {
+INCLUDE('XPCOM');
+LAZY_INCLUDE('DOM', 'HttpInterceptor', 'MediaSniffer');
+
+
+const SHUTDOWN = "profile-before-change";
+const STARTUP = ("nsISessionStore" in CI && typeof(/ /) === "object")
+  ? "sessionstore-windows-restored"
+  : "profile-after-change";
+
+var fg, singleton; // singleton
+
+const SERVICE_CREATE = function() {
+  fg.wrappedJSObject = singleton = fg;
+  
+  if ("nsIChromeRegistrySea" in CI) INCLUDE("SMUninstaller");
+  fg.register();
+  return fg;
+}
+
+fg = {
   OP_ONE: 0, 
   OP_SEL: 1,
   OP_ALL: 2,
@@ -121,47 +134,17 @@ FlashGotService.prototype = {
 ,
   VERSION: VERSION
 ,
-  set fileSpec(f) { // hooked from component registration
-    delete this.fileSpec;
-    this.fileSpec = f;
-    try {
-      f = this._sothinkFile(f.parent);
-      if ((f instanceof CI.nsILocalFile) && f.exists()) {
-        var s = IO.readFile(f);
-        if (s.indexOf(SERVICE_CTRID) > -1) {
-          f.moveTo(f.parent.parent, null);
-          dump("Sothink illegal clone " + f.path + " removed\n");
-        }
-      }
-    } catch(e) {
-      dump(e + "\n");
-    }
-    return this.fileSpec;
+  get dom() {
+    return DOM;
   },
-  _sothinkFile: function(f) {
-    f = f.parent.parent;
-    f.append("{FCAB6FDD-5585-425b-95C1-5ED856F3FD08}");
-    f.append("components");
-    f.append("SWVDService.js");
-    return f;
+  register: function() {
+    const os = this.observerService;
+    os.addObserver(this, SHUTDOWN, false);
+    os.addObserver(this, "xpcom-shutdown", false);
+    os.addObserver(this, STARTUP, false);
+    os.addObserver(this, "private-browsing", false);
+     os.addObserver(this, "browser:purge-session-history", false);
   },
-  sothinkRestore: function() {
-    try {
-      var f = this._sothinkFile(this.home);
-      var sothink = f.parent.parent;
-      sothink.append(f.leafName);
-      if (sothink.exists()) {
-        f = sothink.parent;
-        f.append("components");
-        sothink.moveTo(f, null)
-      }
-      dump("Sothink illegal clone restored in " + f.path + "\n");
-    } catch(e) {
-      dump(e + "\n");
-    }
-  }
-,
-  dom: DOM,
   unregister: function() {
     try {
       const os = this.observerService;
@@ -169,8 +152,10 @@ FlashGotService.prototype = {
       os.removeObserver(this, SHUTDOWN);
       os.removeObserver(this, "xpcom-shutdown");
       os.removeObserver(this, STARTUP);
+      os.removeObserver(this, "private-browsing");
+      os.removeObserver(this, "browser:purge-session-history");
     } catch(ex) {
-      this.log("Error unregistering service as observer: "+ex);
+      this.log("Error unregistering service as observer: " + ex);
     }
   }
 ,
@@ -193,9 +178,7 @@ FlashGotService.prototype = {
           this.cleanup();
           break;
         case STARTUP:
-          this.init();
-          this.interceptor.setup();
-          this.observerService.addObserver(this, "em-action-requested", false);
+          this.delayExec(this.init, 0);
           break;
         case "em-action-requested":
           if ((subject instanceof CI.nsIUpdateItem)
@@ -208,9 +191,24 @@ FlashGotService.prototype = {
             }
           }
         break;
+        case "private-browsing":
+          if (data == "exit") {
+            this.inPrivate = false;
+            this._shredTempFiles();
+          } else if (data == "enter") {
+            this.inPrivate = true;
+          }
+        break;
+        case "browser:purge-session-history":
+          this._shredTempFiles();
+          this._erasePrivatePrefs();
+        break;
       }
     }
   },
+  
+  
+  inPrivate: false,
   uninstalling: false
 ,
   syncPrefs: function(name) {
@@ -236,17 +234,88 @@ FlashGotService.prototype = {
     }
   }
 ,
+  _erasePrivatePrefs: function() {
+    const names = ["redir.rapidshare_com.cookie"];
+    const prefs = this.prefs;
+    for each (var n in names) {
+      try {
+        prefs.clearUserPref(n);
+      } catch (e) {}
+    }
+  }
+,
   
   get defaultDM() {
-    return this.getPref("defaultDM", null);
+    return this.getUPref("defaultDM", null);
   },
   set defaultDM(name) {
-    this.setPref("defaultDM", name);
-    return name;
+    return this.setUPref("defaultDM", name);
   },
 
   get isWindows() {
-    return ("nsIWindowsShellService" in CI) || ("@mozilla.org/winhooks;1" in CC);
+    return this._lazyPlatformCheck("isWin", "WinD");
+  },
+  
+  get isMac() {
+    return this._lazyPlatformCheck("isMac", "UsrDsk");
+  },
+  
+  get isLinux() {
+    return this._lazyPlatformCheck("isLinux", "XDGDesk");
+  },
+  
+  _lazyPlatformCheck: function(prop, folder) {
+    delete this[prop];
+    var is = false;
+    try {
+      is = this.directoryService.get(folder, CI.nsIFile);
+    } catch(e) {}
+    return this[prop] = is;
+  },
+  
+  get directoryService() {
+    delete this.directoryService;
+    return this.directoryService = CC["@mozilla.org/file/directory_service;1"]
+      .getService(CI.nsIProperties);
+  },
+  get cookieManager() {
+    delete this.cookieManager;
+    return this.cookieManager =
+      CC["@mozilla.org/cookiemanager;1"].getService(CI.nsICookieManager2);
+  }
+,
+  
+  get java() {
+    var java;
+    
+    try {
+      java = this.prefs.getComplexValue("java", CI.nsILocalFile);
+    } catch(e) {
+    
+      if (this.isWindows) {
+        java = this.directoryService.get("WinD", CI.nsIFile);
+        java.append("System32");
+        java.append("javaw.exe");
+      } else {
+      
+        java = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
+        
+        if (this.isMac) {
+          java.initWithPath("/usr/bin/java");
+        } else {
+          try {
+            java.initWithPath(DOM.mostRecentBrowserWindow.java.lang.System.getProperty("java.home"));
+            java.append("bin");
+            java.append("java");
+          } catch (e) {
+            java.initWithPath("/usr/bin/java");
+          }
+        }
+      }
+    }
+    
+    delete this.java;
+    return this.java = java;
   }
 ,
 
@@ -311,7 +380,40 @@ FlashGotService.prototype = {
        this._httpServer=new FlashGotHttpServer(this)
       :this._httpServer;
   }
-
+,
+  createDownloadFile: function(parentFile, fileSpec, overwrite) {
+    
+    if (fileSpec instanceof CI.nsIURL) {
+     
+      fileSpec = fileSpec.fileName || fileSpec.filePath.replace(/.*?([^\/]*)\/?$/, '$1') || fileSpec.query || fileSpec.host;
+      try {
+        fileSpec = decodeURIComponent(fileSpec);
+      } catch(e) {
+        fileSpec = unescape(fileSpec);
+      }
+      fileSpec = fileSpec.replace(/[\u0000-\u001f\/]/g, '_');
+    } else if (fileSpec instanceof CI.nsIURI) {
+      return null;
+    }
+    
+    if (!fileSpec) return null;
+    
+    var file = parentFile.clone();
+    file.append(fileSpec);
+            
+    if (!overwrite || file.exists() && file.isDirectory()) {
+      for (;;) {
+        if(!file.exists()) {
+          file.create(0, 0644);
+          break;
+        } else { // rename
+          var m = file.leafName.match(/(.*?)(?:\((\d+)\))?(\.[^\.]+$|$)/);
+          file.leafName = m[1] + "(" + ((m[2] && parseInt(m[2]) || 0) + 1) + ")" + m[3]; 
+        }
+      }
+    }
+    return file;
+  }
 ,
   download: function(links, opType, dmName) {
     
@@ -361,7 +463,7 @@ FlashGotService.prototype = {
           this.extensions.join("|").replace(/[^\w-|]/,"") + 
           ")\\b", "i") : null;
     
-    var logMsg = "Processing "+links.length+" links ";
+    var logMsg = "Processing " + links.length + " links ";
     if (this.logEnabled && typeof(links.startTime) == "number") {
       logMsg += "scanned in ms" + (Date.now() - links.startTime);
     }
@@ -371,9 +473,6 @@ FlashGotService.prototype = {
     if (!links.startTime) links.startTime = Date.now();
     const pg = links.progress;
     
-    const escapeCheckNo=/(%[0-9a-f]{2,4})/i;
-    const escapeCheckYes=/[\s]+/;
-    
     var len = links.length;
     
     var filters = null;
@@ -400,49 +499,60 @@ FlashGotService.prototype = {
     pg.update(10);
     
     const stripHash = dm.getPref("stripHash", false);
+    var cs = links.document && links.document.characterSet || null;
     
     var j, l, href, ol, pos1, pos2;
     for (j = 0; j < len; j++) {
       l = links[j];
+      
+      href = l.href;
+      
+      if (/^javascript:/i.test(href)) {
+        try {
+          l.href = href = href.match(/(['"])(https?:\/\/.*?)\1/)[2];
+        } catch(e) {
+          continue;
+        }
+        if (!href) continue;
+      }
+      
+      if (filters && !filters.doFilter(href)) continue;
+      
       if (l.description) l.description = l.description.replace(/\s+/g, ' ');
       l._pos = j;
-      href = l.href;
-      if ((!filters) || filters.doFilter(href)) {
-        ol = map[href];
-        if (ol) { // duplicate, keep the longest description
-          if (ol.description.length < l.description.length) {
-            map[href] = l;
-            l.href = ol.href; // keep sanitizations
-          }
-        } else {
+      
+      ol = map[href];
+      if (ol) { // duplicate, keep the longest description
+        if (ol.description.length < l.description.length) {
           map[href] = l;
-          
-          // encoding checks
-          try {
-            if (encodedURLs) { 
-              if (escapeCheckYes.test(href) || !escapeCheckNo.test(href)) { 
-                href = encodeURI(href);
-              }
-              // workaround for malformed hash urls
-             
-              while ((pos1 = href.indexOf("#")) > -1 // has fragment?
-                && href[pos1 + 1] != "!" // skip metalinks!
-                && (href.indexOf("?") > pos1 || pos1 != href.lastIndexOf('#')) // fragment before query or double fragment ? 
-              ) {
-                href = href.substring(0, pos1) + '%23' + href.substring(pos1 + 1);
-              }
-              
-              l.href = href;
-            } else {  
-              l.href = decodeURI(href);
+          l.href = ol.href; // keep sanitizations
+        }
+      } else {
+        map[href] = l;
+        
+        // encoding checks
+        try {
+          if (encodedURLs) { 
+            href = IOS.newURI(href, cs, null).asciiSpec; // punycode + % encoding
+            // workaround for malformed hash urls
+           
+            while ((pos1 = href.indexOf("#")) > -1 // has fragment?
+              && href[pos1 + 1] != "!" // skip metalinks!
+              && (href.indexOf("?") > pos1 || pos1 != href.lastIndexOf('#')) // fragment before query or double fragment ? 
+            ) {
+              href = href.substring(0, pos1) + '%23' + href.substring(pos1 + 1);
             }
-            if (stripHash) l.href = l.href.replace(/#.*/g, '');
             
-          } catch(e) {
-            dump("Problem "
-              + ( encodedURLs ? "escaping" : "unescaping")
-              + " URL " + href + ": "+ e.message + "\n");
+            l.href = href;
+          } else {  
+            l.href = decodeURI(href);
           }
+          if (stripHash) l.href = l.href.replace(/#.*/g, '');
+          
+        } catch(e) {
+          dump("Problem "
+            + ( encodedURLs ? "escaping" : "unescaping")
+            + " URL " + href + ": "+ e.message + "\n");
         }
       }
     }
@@ -466,9 +576,9 @@ FlashGotService.prototype = {
   },
   
   get RedirectContext() {
-    delete this.__proto__.RedirectContext;
+    delete this.RedirectContext;
     INCLUDE('RedirectContext');
-    return this.__proto__.RedirectContext = RedirectContext;
+    return this.RedirectContext = RedirectContext;
   },
   _processRedirects: function(links, opType, dm) {
     links.progress.update(30);
@@ -497,9 +607,10 @@ FlashGotService.prototype = {
     if (links.length > 1) {
       dm.log("Sorting again "+links.length+" links");
       links.sort(function(a,b) {
-        a=a._pos; b=b._pos;
-        return a>b?1:a<b?-1:0;
+        a = a._pos; b = b._pos;
+        return a > b ? 1 : a < b ? -1: 0;
       });
+      opType = this.OP_SEL;
     }
     
     this._addQsSuffix(links);
@@ -541,21 +652,41 @@ FlashGotService.prototype = {
   _addAuthInfo: function(links) {
     const httpAuthManager = CC['@mozilla.org/network/http-auth-manager;1']
                               .getService(CI.nsIHttpAuthManager);
-    var uri;
-    var udom = {};
-    var uname = {};
-    var upwd = {};
-    var l;
+    const seen = {};
+    const udom = {}, uname = {}, upwd = {};
+    var uri, l;
     for (var j = links.length; j-- > 0;) {
       l = links[j];
+
+      uri = IOS.newURI(l.href, null, null);
+      l.userPass = uri.userPass;
+      if (l.userPass && l.userPass.indexOf(":") > -1) {
+        continue;
+      }
       try {
-        uri = IOS.newURI(l.href, null, null);
-        if (l.userPass && l.userPass.indexOf(":") > -1) continue;
         httpAuthManager.getAuthIdentity(uri.scheme, uri.host, uri.port < 0 ? (uri.scheme == "https" ? 443 : 80) : uri.port, null, null, uri.path, udom, uname, upwd);
-        this.log("Authentication data for " + uri + " added.");
-        l.href = uri.scheme + "://" + uname.value + ":" + upwd.value + "@" + 
-                 uri.host + (uri.port < 0 ? "" : (":" + uri.port)) + uri.spec.substring(uri.prePath.length);
-      } catch(e) {}
+      } catch(e) {
+        
+        if (uri.scheme !== "ftp" || !uri.username)
+          continue;
+
+        const prompter = CC["@mozilla.org/embedcomp/window-watcher;1"].getService(CI.nsIWindowWatcher)
+          .getNewAuthPrompter(null).QueryInterface(CI.nsIAuthPrompt);
+        var port = (uri.port < 0 || uri.port == 21 ? "" : ":" + uri.port);
+        var realm = uri.scheme + "://" + uri.username + "@" + uri.host + port;
+        upwd.value = (realm in seen) ? seen[realm] : ''; 
+        if (!(upwd.value || 
+            prompter.promptPassword("Password", realm + " password", realm, prompter.SAVE_PASSWORD_NEVER, upwd))
+          )
+          continue;
+        
+        uname.value = uri.username;
+        seen[realm] = upwd.value;
+      }
+      l.userPass = uname.value + ":" + upwd.value;
+      fg.log("Authentication data " + l.userPass + " for " + l.href + " added.");
+      l.href = uri.scheme + "://" + l.userPass + "@" + 
+               uri.host + (uri.port < 0 ? "" : (":" + uri.port)) + uri.spec.substring(uri.prePath.length);
     }
   },
   _processMetalinks: function(links) {
@@ -577,8 +708,8 @@ FlashGotService.prototype = {
          key = couple[0].toLowerCase();
          switch (key) {
            case "md5": case "sha1":
-             l[key] = couple[1];
-             break;
+            l[key] = couple[1];
+            break;
            case "metalink":
             if (/^(https?|ftp):/i.test(couple[1])) {
               l.metalinks.push(couple[1]);
@@ -591,26 +722,32 @@ FlashGotService.prototype = {
   }
 ,
 
+  _timers: [],
   delayExec: function(callback, delay) {
-    const timer = CC["@mozilla.org/timer;1"].createInstance(
-      CI.nsITimer);
-     var args = Array.prototype.slice.call(arguments, 2);
-     timer.initWithCallback({ 
-         notify: this.delayedRunner,
-         context: { callback: callback, args: args, self: this }
-      },  delay || 1, 0);
+    var timer = CC["@mozilla.org/timer;1"].createInstance(CI.nsITimer);
+    timer.initWithCallback({ 
+        notify: this.delayedRunner,
+        context: { callback: callback, args: Array.prototype.slice.call(arguments, 2), self: this }
+      }, delay || 1, 0);
+    this._timers.push(timer);
   },
-  delayedRunner: function() {
+  
+  
+  delayedRunner: function(timer) {
     var ctx = this.context;
     try {
       ctx.callback.apply(ctx.self, ctx.args);
-     } catch(e) {
-      fg.log("Delayed Runner error: " + e + ", " + e.stack);
-     }
-     finally {
-       ctx.args = null;
-       ctx.callback = null;
-     }
+    } catch(e) {
+     fg.log("Delayed Runner error: " + e + ", " + e.stack);
+    } finally {
+      ctx.args = null;
+      ctx.callback = null;
+
+      var tt = fg._timers;
+      var pos = tt.indexOf(timer);
+      if (pos > -1) tt.splice(pos, 1);
+      timer.cancel();
+    }
   }
 ,
   yield: function() {
@@ -629,8 +766,8 @@ FlashGotService.prototype = {
 ,
   
   get prefService() {
-    delete this.__proto__.prefService;
-    return this.__proto__.prefService = CC["@mozilla.org/preferences-service;1"].getService(CI.nsIPrefService);
+    delete this.prefService;
+    return this.prefService = CC["@mozilla.org/preferences-service;1"].getService(CI.nsIPrefService);
   }
 ,
   savePrefs: function() {
@@ -653,8 +790,8 @@ FlashGotService.prototype = {
     return def;
   }
 ,
-  setPref: function(name,value) {
-    const prefs=this.prefs;
+  setPref: function(name, value) {
+    const prefs = this.prefs;
     switch (typeof(value)) {
       case "string":
           prefs.setCharPref(name,value);
@@ -670,44 +807,92 @@ FlashGotService.prototype = {
     }
   }
 ,
+  getUPref: function(name, def) {
+    try {
+      return this.prefs.getComplexValue(name, CI.nsISupportsString).data;
+    } catch(e) {
+      return def;
+    }
+  },
+  setUPref: function(name, value) {
+    var str = CC["@mozilla.org/supports-string;1"]
+        .createInstance(CI.nsISupportsString);
+    str.data = value;
+    this.prefs.setComplexValue(name, CI.nsISupportsString, str);
+    return value;
+  },
+
   get getString() {
-    delete this.__proto__.getString;
+    delete this.getString;
     INCLUDE('Strings');
     const ss = new Strings("flashgot");
-    return this.__proto__.getString = function(name, parms) { return ss.getString(name, parms) };
+    return this.getString = function(name, parms) { return ss.getString(name, parms) };
   },
 
-  _logFile: null,
   get logFile() {
-    if (this._logFile==null) {
-      this._logFile=this.profDir.clone();
-      this._logFile.append("flashgot.log");
-    }
-    return this._logFile;
+    delete this.logFile;
+    var logFile = this.profDir.clone();
+    logFile.append("flashgot.log");
+    return this.logFile = logFile;
   }
 ,
   logStream: null,
   logEnabled: false,
+  logInit: function() {
+    const logFile = this.logFile;
+    const stream = CC["@mozilla.org/network/file-output-stream;1"]
+      .createInstance(CI.nsIFileOutputStream);
+    stream.init(logFile, 0x02 | 0x08 | 0x10, 0600, 0);
+    const bufferedStream = CC["@mozilla.org/network/buffered-output-stream;1"]
+      .createInstance(CI.nsIBufferedOutputStream);
+    bufferedStream.init(stream, 32768);
+    const charStream = CC["@mozilla.org/intl/converter-output-stream;1"]
+      .createInstance(CI.nsIConverterOutputStream);
+    charStream.init(bufferedStream, "UTF-8", 0, 0);
+    charStream.writeString("*** FlashGot " + this.VERSION + " started at " + new Date().toGMTString() + " ***\n");
+    this.logStream = charStream;
+    this.logFlush = function() {
+      if (this.logStream) {
+        this.logStream.flush();
+        bufferedStream.flush();
+      }
+    };
+  },
+  
+  logFlush: function() {
+    this.logStream._handOver();
+    this.logFlush();
+  },
+  
+  logStream: { // we buffer in ram until we reach 16KB or we get flushed, in order to reduce startup footprint
+    _buf: "",
+    writeString: function(msg) {
+      this._buf += msg;
+      if (this._buf.length > 16384) this._handOver();
+    },
+    flush: function() {
+      fg.logFlush();
+    },
+    close: function() {
+      this._handOver();
+      fg.logStream.close();
+    },
+    _handOver: function() {
+      fg.logInit();
+      fg.logStream.writeString(this._buf);
+    }
+  },
+  
   log: function(msg) {
     if (this.logEnabled) {
       try {
-        if (!this.logStream) {
-          const logFile=this.logFile;
-          const logStream=CC["@mozilla.org/network/file-output-stream;1"
-            ].createInstance(CI.nsIFileOutputStream );
-          logStream.init(logFile, 0x02 | 0x08 | 0x10, 0600, 0 );
-          this.logStream=logStream;
-          const header="*** Log start at "+new Date().toGMTString()+"\n";
-          this.logStream.write(header,header.length);
+        if (msg !== null && !this.inPrivate) {
+          msg += "\n";
+          this.logStream.writeString(msg);
         }
-        
-        if (msg!=null) {
-          msg+="\n";
-          this.logStream.write(msg,msg.length);
-        }
-        this.logStream.flush();
+
       } catch(ex) {
-        dump(ex.message+"\noccurred logging this message:\n"+msg);
+        dump(ex.message + "\noccurred logging this message:\n" + msg);
       }
     }
   }
@@ -726,7 +911,7 @@ FlashGotService.prototype = {
         }
       }
       if (this.logFile) this.logFile.remove(true);
-      this.logStream=null;
+      this.logStream = null;
       this.log(null);
     } catch(ex) { dump(ex.message); }
   } 
@@ -741,73 +926,83 @@ FlashGotService.prototype = {
   }
 ,
   getBrowserWindow: function(document) {
-    if (!document) return null;
-    var w = DOM.getChromeWindow(document.defaultView.top);
+    var w = document && document.defaultView && DOM.getChromeWindow(document.defaultView.top) || DOM.mostRecentBrowserWindow;
     return w.wrappedJSObject || w; 
   }
 ,
   get prefs() {
-    delete this.__proto__.prefs;
-    return this.__proto__.prefs = this.prefService.getBranch("flashgot.").QueryInterface(CI.nsIPrefBranchInternal);
+    delete this.prefs;
+    return this.prefs = this.prefService.getBranch("flashgot.").QueryInterface(CI.nsIPrefBranchInternal);
   }
 ,
-  DMS: null,
-  tmpDir: null,
-  profDir: null,
+  hasDMS: false,
+  get DMS() {
+    delete this.DMS;
+    this.hasDMS = true;
+    return this.DMS = this.checkDownloadManagers(true, false);
+  },
+  
+  get tmpDir() {
+    
+    function prepareTmp(t, create) {
+      if (!(create || t.exists())) throw "Temporary dir " + t.path + " doesn't exist.";
+      
+      t.append("flashgot." + encodeURI(fg.profDir.leafName).replace(/%/g,"_"));
+      if (t.exists()) {
+       if (!(t.isDirectory() && t.isWritable())) t.createUnique(1, 0700);
+      } else {
+        t.create(1, 0700);
+      } 
+      return t;
+    }
+    
+    delete this.tmpDir;
+    try {
+      return this.tmpDir = prepareTmp(this.prefs.getComplexValue("tmpDir", CI.nsILocalFile), false);
+    } catch (e) {}
+    
+    return this.tmpDir = prepareTmp(this.directoryService.get("TmpD", CI.nsILocalFile), true);
+   
+  },
+  get profDir() {
+    delete this.profDir;
+    return this.profDir = this.directoryService.get("ProfD", CI.nsIFile);
+  },
+  
   _initialized: false,
   init: function() {
     if (this._initialized) return;
-    
-    if (this.smUninstaller) this.smUninstaller.check();
-    
-    const os = this.observerService;
-    this.interceptor = new HttpInterceptor();
-    os.addObserver(this.interceptor, "http-on-modify-request", true);
 
-    if (this.getPref("media.enabled", true)) {
-      os.addObserver(MediaSniffer, "http-on-examine-response", true);
-      CC['@mozilla.org/docloaderservice;1'].getService(CI.nsIWebProgress)
-        .addProgressListener(MediaSniffer, CI.nsIWebProgress.NOTIFY_STATE_NETWORK);
-    }
+    if (this.smUninstaller) this.smUninstaller.check();
+  
+    const os = this.observerService;
+    
+    os.addObserver(this, "em-action-requested", false);
     
     try {
       const startTime = Date.now();
-      
-      const fileLocator = CC["@mozilla.org/file/directory_service;1"].getService(
-        CI.nsIProperties);
-      
-      var profDir = this.profDir = fileLocator.get("ProfD", CI.nsIFile);
-      
-      function prepareTmp(t) {
-        t.append("flashgot." + encodeURI(profDir.leafName).replace(/%/g,"_"));
-        if (t.exists()) {
-         if (!t.isDirectory()) t.createUnique(1, 0700);
-        } else {
-          t.create(1,0700);
-        }
-        return t;
-      }
-      
-      try {
-        this.tmpDir = prepareTmp(this.prefs.getComplexValue("tmpDir", CI.nsILocalFile));
-      } catch(ex) {
-        this.tmpDir = prepareTmp(fileLocator.get("TmpD", CI.nsILocalFile));
-      }
        
       this.prefs.addObserver("", this, false);
       this.syncPrefs();
       
       this.log("Per-session init started");
-        
-      this._setupLegacyPrefs();
 
-      this.DMS = this.checkDownloadManagers(true, false);
+      this.interceptor = new HttpInterceptor();
+      os.addObserver(this.interceptor, "http-on-modify-request", true);
+      if (this.getPref("media.enabled", true)) {
+        os.addObserver(MediaSniffer, "http-on-examine-response", true);
+        os.addObserver(MediaSniffer, "http-on-examine-cached-response", true);
+        CC['@mozilla.org/docloaderservice;1'].getService(CI.nsIWebProgress)
+          .addProgressListener(MediaSniffer, CI.nsIWebProgress.NOTIFY_STATE_NETWORK);
+      }
+      this.interceptor.setup();
+    
       this.log("Per-session init done in " + (Date.now() - startTime) + "ms");
     } catch(initEx) {
       this._initException = initEx;
       try { this.log(initEx); } catch(e) {}
     }
-    this._initialized=true; 
+    this._initialized = true; 
   },
 
   dispose: function() {
@@ -815,17 +1010,19 @@ FlashGotService.prototype = {
     const os = this.observerService;
     os.removeObserver(this.interceptor, "http-on-modify-request");
     this.interceptor.dispose();
-    this.interceptor = null;
+    delete this.interceptor;
     try {
       os.removeObserver(MediaSniffer, "http-on-examine-response");
+      os.removeObserver(MediaSniffer, "http-on-examine-cached-response");
       CC['@mozilla.org/docloaderservice;1'].getService(CI.nsIWebProgress)
         .removeProgressListener(MediaSniffer);
     } catch(e) {}
     
-    this._initialized=false;
+    this._initialized = false;
   }
 ,
   createCustomDM: function(name) {
+    this.DMS; // kickstart if late
     const dm = new FlashGotDMCust(name);
     if (name && name.length) {
       FlashGotDMCust.persist(this);
@@ -835,17 +1032,17 @@ FlashGotService.prototype = {
     return dm;
   }
 ,
- removeCustomDM: function(name) {
-   const dms = FlashGotDM.dms;
-   for (var j = dms.length; j-->0;) {
-     if (dms[j].custom && dms[j].name == name) {
-       dms.splice(j, 1);
-       delete dms[name];
-     }
-   }
-   FlashGotDMCust.persist(this);
-   this.checkDownloadManagers(false, false);
- }
+  removeCustomDM: function(name) {
+    const dms = FlashGotDM.dms;
+     for (var j = dms.length; j-->0;) {
+       if (dms[j].custom && dms[j].name == name) {
+         dms.splice(j, 1);
+         delete dms[name];
+      }
+    }
+    FlashGotDMCust.persist(this);
+    this.checkDownloadManagers(false, false);
+  }
 ,
   sortDMS: function() {
     FlashGotDM.dms.sort(function(a,b) { 
@@ -856,6 +1053,9 @@ FlashGotService.prototype = {
   }
 , 
   checkDownloadManagers: function(init, detect) {
+    INCLUDE("DMS");
+    
+    var t = Date.now();
     
     if (init || detect) FlashGotDM.init(this);
     
@@ -867,7 +1067,8 @@ FlashGotService.prototype = {
     detect = detect || this.getPref("detect.auto", true);
  
     var j, dm;
-    var cache = this.getPref("detect.cache", "").split(",");
+    var cache = this.getUPref('detect.cache', '').split(',');
+    
     var cacheLen = cache.length;
     if (!detect) {
       for (j = dms.length; j-- > 0;) {
@@ -877,21 +1078,20 @@ FlashGotService.prototype = {
       var name;
       for (j = cache.length; j-- > 0;) {
         name = cache[j];
-        if (name.length && typeof(dm = dms[name])=="object" && dm.name == name) {
+        if (name.length && typeof(dm = dms[name]) == "object" && dm.name == name) {
           dm._supported = true;
         }
       }
     }
     
     cache = [];
-    var exclusive;
+
     var firstSupported = null;
     for (j = dms.length; j-- >0;) {
       dm = dms[j];
       if (dm.supported) {
         dms.found = true;
         cache.push(firstSupported = dm);
-        if (dm.exclusive) exclusive = true;
       } else {
         this.log("Warning: download manager " + dm.name + " not found");
         if (defaultDM == dm.name) {
@@ -901,7 +1101,7 @@ FlashGotService.prototype = {
       }
     }
     
-    this.setPref("detect.cache", cache.map(function(dm) { return dm.name; }).join(","));
+    this.setUPref("detect.cache", cache.map(function(dm) { return dm.name; }).join(","));
     
     if (cacheLen > 0 && cache.length > cacheLen && defaultDM && !dms[defaultDM].autoselect) { // new DM added
       defaultDM = null;
@@ -917,19 +1117,40 @@ FlashGotService.prototype = {
       this.log("Serious warning! no supported download manager found...");
     }
     
-    if (exclusive) {
-      for (j = dms.length; j-->0;) {
-        if (!(dms[j].custom || dms[j].supported) ) {
-          dms.splice(j,1);
-        }
-      }
-    }
+    this.log("Download managers detection done in " + (Date.now() - t) + "ms");
     
     return dms;
   }
 ,
   _cleaningup: false
 ,
+  _shredTempFiles: function() {
+    try {
+      var files = [];
+      var fen = this.tmpDir.directoryEntries;
+      var f;
+      while (fen.hasMoreElements()) {
+        f = fen.getNext();
+        if (f instanceof CI.nsIFile) files.push(f);
+      }
+      if (files.length) this.shredFiles(files);
+    } catch(e) {
+      dump("Error shredding temp files: " + e + "\n");
+    }
+  },
+  
+  doomPrivateFile: function(f) {
+    if (this.inPrivate || this.getPref("shredding"))
+      this.delayExec(function() { fg.shredFiles(f); }, 5000);
+  },
+  
+  shredFiles: function(files) {
+    INCLUDE("Shredder");
+    if (!("push" in files)) files = [files];
+    Shredder.shred(files);
+  },
+  
+  
   cleanup: function() {
     if (this._cleaningup) return;
     try {
@@ -947,22 +1168,27 @@ FlashGotService.prototype = {
       
       if (this.tmpDir && this.tmpDir.exists()) {
         try {
-          this.tmpDir.remove(true);
+          if ((this.inPrivate || this.getPref("shredding")))
+          {
+            this._shredTempFiles(); 
+          } else {
+            this.tmpDir.remove(true);
+          }
         } catch(eexx) {
           this.log("Can't remove " + this.tmpDir.path + ", maybe still in use: " + eexx);
         }
       }
       this._bundle = null;
       this.log("Cleanup done");
-      if (this._logFile) try {
+      if (this.logFile) try {
         if (this.logStream) this.logStream.close();
-        var maxLogSize = Math.max(Math.min(this.getPref('maxLogSize',100000),1000000),50000);
+        var maxLogSize = Math.max(Math.min(this.getPref('maxLogSize', 100000), 1000000), 50000);
         const logFile = this.logFile;
         const logSize = logFile.fileSize;
         const logBak = logFile.clone();
-        logBak.leafName = logBak.leafName+".bak";
+        logBak.leafName = logBak.leafName + ".bak";
         if (logBak.exists()) logBak.remove(true);
-          
+
         if (this.uninstalling) {
           logFile.remove(false);
         } else if (logSize > maxLogSize) { // log rotation
@@ -971,7 +1197,7 @@ FlashGotService.prototype = {
           logFile.copyTo(logBak.parent, logBak.leafName);
           const is=CC['@mozilla.org/network/file-input-stream;1'].createInstance(
             CI.nsIFileInputStream);
-          is.init(logBak,0x01, 0400, null);
+          is.init(logBak, 0x01, 0400, null);
           is.QueryInterface(CI.nsISeekableStream);
           is.seek(CI.nsISeekableStream.NS_SEEK_END,-maxLogSize);
           const sis=CC['@mozilla.org/scriptableinputstream;1'].createInstance(
@@ -980,20 +1206,20 @@ FlashGotService.prototype = {
           var buffer;
           var content="\n";
           var logStart=-1;
-          while ((buffer=sis.read(5000))) {
-            content+=buffer;
-            if ((logStart=content.indexOf("\n*** Log start at "))>-1) { 
+          while ((buffer = sis.read(5000))) {
+            content += buffer;
+            if ((logStart=content.indexOf("\n*** Log start at ")) > -1) { 
               content=content.substring(logStart);
               break;
             }
             content=buffer;
           }
-          if (logStart>-1) {
+          if (logStart > -1) {
              const os=CC["@mozilla.org/network/file-output-stream;1"].createInstance(
               CI.nsIFileOutputStream);
             os.init(logFile,0x02 | 0x08 | 0x20, 0700, 0);
             os.write(content,content.length);
-            while ((buffer=sis.read(20000))) {
+            while ((buffer = sis.read(20000))) {
               os.write(buffer,buffer.length);
             } 
             os.close();
@@ -1023,25 +1249,66 @@ FlashGotService.prototype = {
   get lookupMethod() {
     return this._lookupMethod?this._lookupMethod:(this._lookupMethod = 
       (Components.utils && Components.utils.lookupMethod)
-        ?Components.utils.lookupMethod:Components.lookupMethod);
-  }
-,
-  _setupLegacyPrefs: function() {
-    // check and move flashgot.flashgot.dmsopts branch from previous bug
-    try {
-      for each (var key in this.prefs.getChildList("flashgot.dmsopts.", {})) {
-        this.setPref(key.replace(/^flashgot\./, ""), this.getPref(key));
-      }
-      this.prefs.deleteBranch("flashgot.dmsopts.");
-    } catch(e) {
-      dump(e + "\n");
-    }
+        ? Components.utils.lookupMethod:Components.lookupMethod);
   }
 ,
   showDMSReference: function() {
-    this.getWindow().open("http://www.flashgot.net/dms","_blank");
+    this.getWindow().open("http://flashgot.net/dms","_blank");
   }
 , 
-  dirtyJobsDone: false
+
+  firstRun: false,
+  versionChecked: false,
+  checkVersion: function() {
+    if (this.versionChecked) return;
+    
+    this.versionChecked = true;
+    
+    const ver =  this.VERSION;
+    const prevVer = this.getPref("version", "");
+    if ((this.firstRun = prevVer !== ver)) {
+       if (!prevVer) {
+        // first run ever
+        this.setPref("media.dm", this.getString("dm.builtIn"));
+      }
+      this.setPref("version", ver);
+      this.savePrefs();
+    }
+    
+    const betaRx = /(?:a|alpha|b|beta|pre|rc)\d*$/; // see http://viewvc.svn.mozilla.org/vc/addons/trunk/site/app/config/constants.php?view=markup#l431
+    if (prevVer.replace(betaRx, "") != ver.replace(betaRx, "")) {
+
+      if (this.getPref("firstRunRedirection", true)) {
+        const name = EXTENSION_NAME;
+        const domain = name.toLowerCase() + ".net";
+        IOS.newChannel("http://" + domain + "/-", null, null).asyncOpen({ // DNS prefetch
+          onStartRequest: function() {},
+          onStopRequest: function() {
+            var browser = DOM.mostRecentBrowserWindow.getBrowser();
+            if (typeof(browser.addTab) != "function") return;
+           
+            
+            var url = "http://" + domain + "/?ver=" + ver;
+            var hh = "X-IA-Post-Install: " + name + " " + ver;
+            if (prevVer) {
+              url += "&prev=" + prevVer;
+              hh += "; updatedFrom=" + prevVer;
+            }
+            hh += "\r\n";
+            
+            var hs = CC["@mozilla.org/io/string-input-stream;1"] .createInstance(CI.nsIStringInputStream);
+            hs.setData(hh, hh.length); 
+            
+            
+            var b = (browser.selectedTab = browser.addTab()).linkedBrowser;
+            b.stop();
+            b.webNavigation.loadURI(url, CI.nsIWebNavigation.LOAD_FLAGS_NONE, null, null, hs);
+            
+          },
+          onDataAvailable: function() {}
+        }, {});
+      }
+    }
+  }
 }
 
